@@ -83,10 +83,7 @@ class PostgresInstaller extends DatabaseInstaller {
 
 	function submitConnectForm() {
 		// Get variables from the request
-		$newValues = $this->setVarsFromRequest( array(
-			'wgDBserver', 'wgDBport', 'wgDBname', 'wgDBmwschema',
-			'_InstallUser', '_InstallPassword'
-		) );
+		$newValues = $this->setVarsFromRequest( array( 'wgDBserver', 'wgDBport', 'wgDBname', 'wgDBmwschema' ) );
 
 		// Validate them
 		$status = Status::newGood();
@@ -98,12 +95,6 @@ class PostgresInstaller extends DatabaseInstaller {
 		if ( !preg_match( '/^[a-zA-Z0-9_]*$/', $newValues['wgDBmwschema'] ) ) {
 			$status->fatal( 'config-invalid-schema', $newValues['wgDBmwschema'] );
 		}
-		if ( !strlen( $newValues['_InstallUser'] ) ) {
-			$status->fatal( 'config-db-username-empty' );
-		}
-		if ( !strlen( $newValues['_InstallPassword'] ) ) {
-			$status->fatal( 'config-db-password-empty', $newValues['_InstallUser'] );
-		}
 
 		// Submit user box
 		if ( $status->isOK() ) {
