@@ -104,7 +104,7 @@ class SpecialPasswordReset extends FormSpecialPage {
 	}
 
 	protected function getDisplayFormat() {
-		return 'vform';
+		return 'ooui';
 	}
 
 	public function alterForm( HTMLForm $form ) {
