@@ -91,7 +91,7 @@ class NewParserTest extends MediaWikiTestCase {
 		);
 		$tmpGlobals['wgForeignFileRepos'] = array();
 		$tmpGlobals['wgDefaultExternalStore'] = array();
-		$tmpGlobals['wgEnableParserCache'] = false;
+		$tmpGlobals['wgParserCacheType'] = CACHE_NONE;
 		$tmpGlobals['wgCapitalLinks'] = true;
 		$tmpGlobals['wgNoFollowLinks'] = true;
 		$tmpGlobals['wgNoFollowDomainExceptions'] = array();
@@ -106,7 +106,6 @@ class NewParserTest extends MediaWikiTestCase {
 		$tmpGlobals['wgAdaptiveMessageCache'] = true;
 		$tmpGlobals['wgUseDatabaseMessages'] = true;
 		$tmpGlobals['wgLocaltimezone'] = 'UTC';
-		$tmpGlobals['wgDeferredUpdateList'] = array();
 		$tmpGlobals['wgGroupPermissions'] = array(
 			'*' => array(
 				'createaccount' => true,
@@ -160,10 +159,10 @@ class NewParserTest extends MediaWikiTestCase {
 		$this->djVuSupport = new DjVuSupport();
 		// Tidy support
 		$this->tidySupport = new TidySupport();
+		$tmpGlobals['wgTidyConfig'] = null;
 		$tmpGlobals['wgUseTidy'] = false;
-		$tmpGlobals['wgAlwaysUseTidy'] = false;
 		$tmpGlobals['wgDebugTidy'] = false;
-		$tmpGlobals['wgTidyConf'] = $IP . '/includes/tidy.conf';
+		$tmpGlobals['wgTidyConf'] = $IP . '/includes/tidy/tidy.conf';
 		$tmpGlobals['wgTidyOpts'] = '';
 		$tmpGlobals['wgTidyInternal'] = $this->tidySupport->isInternal();
 
@@ -185,6 +184,8 @@ class NewParserTest extends MediaWikiTestCase {
 		$wgNamespaceAliases['Image'] = $this->savedWeirdGlobals['image_alias'];
 		$wgNamespaceAliases['Image_talk'] = $this->savedWeirdGlobals['image_talk_alias'];
 
+		MWTidy::destroySingleton();
+
 		// Restore backends
 		RepoGroup::destroySingleton();
 		FileBackendGroup::destroySingleton();
@@ -454,6 +455,7 @@ class NewParserTest extends MediaWikiTestCase {
 			$GLOBALS[$var] = $val;
 		}
 
+		MWTidy::destroySingleton();
 		MagicWord::clearCache();
 
 		# The entries saved into RepoGroup cache with previous globals will be wrong.
